/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.service;

import org.ow2.jasmine.event.beans.JasmineEventEB;
import org.ow2.jasmine.monitoring.eos.monitoring.DataBaseEventExtractor;
import org.ow2.jasmine.monitoring.eos.monitoring.EventData;
import org.ow2.jasmine.monitoring.eos.monitoring.Serie;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * Extractor for Eventswitch database
 * @author durieuxp
 */
public class DatabaseExtractor extends Extractor {

    private static SimpleDateFormat format;

    // TODO configure
    private int nbrecordread = 12000;

    // TODO configure
    private int nbmillisecwait = 10000;

    // ------------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------------

    /**
     * Constructor for DashboardDataCollector
     */
    public DatabaseExtractor(DashboardDataCollector dbc) {
        super(dbc);
        format = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
    }


    // ------------------------------------------------------------------------
    // Interface
    // ------------------------------------------------------------------------

    public long getStartTime() throws Exception {
        DataBaseEventExtractor dataBaseEventExtractor = new DataBaseEventExtractor();
        JasmineEventEB elem = null;
        try {
            elem = dataBaseEventExtractor.getFirstEvent();
            if (elem == null) {
                logger.warn("Nothing in database yet");
                return new Date().getTime();
            }
        } catch (Exception e) {
            logger.error("Cannot read database: " + e);
            throw(e);
        }
        return elem.getTimestamp().getTime();
    }

    // ------------------------------------------------------------------------
    // Thread implementation
    // ------------------------------------------------------------------------

    /**
     * Run method called when the thread is started.
     */
    public void run() {

        // Use a Set to avoid duplicate elements
        Set<String> servers = new HashSet<String>();
        Set<String> probes = new HashSet<String>();
        for (Serie element : dbc.getSeries()) {
            probes.add(element.getYcol());
            if (element.getServer() != null) {
                servers.add(element.getServer());
            }
        }

        // Connect to EJB
        DataBaseEventExtractor dataBaseEventExtractor = new DataBaseEventExtractor();

        // Get events from database
        Date fromDate = from;
        while (! stopWanted) {
            JasmineEventEB[] events = null;
            logger.debug("replay from database from " + fromDate + " to " + to);
            try {
                events = dataBaseEventExtractor.getEvents(servers, probes, fromDate, to, nbrecordread);
                if (events == null) {
                    if (to.getTime() < System.currentTimeMillis()) {
                        // We have reached the time limit
                        break;
                    }
                    logger.debug("Waiting for new events from database");
                    Thread.sleep(nbmillisecwait);
                    continue;
                }
            } catch (Exception e) {
                logger.error("Cannot read database: " + e);
                break;
            }

            long timeLimit = to.getTime() - 120000;
            long lasttimestamp = 0;

            // return only events matching one of the Series
            for (JasmineEventEB elem : events) {
                lasttimestamp = elem.getTimestamp().getTime();

                // Work around for MONITORING-309 (Oracle only) :
                // Make sure we never have a null value, but an empty string instead.
                String sValue = "";
                if (elem.getValue() != null) {
                    sValue = elem.getValue().toString();
                }

                int index = elem.getProbe().lastIndexOf(":");
                String mbean = elem.getProbe().substring(0, index);
                String attr = elem.getProbe().substring(index + 1);

                // Find series accepting this event.
                for (Serie curser : dbc.getSeries()) {

                    // Check Attribute
                    if (! attrmatch(attr, curser.getYcol())) {
                        continue;
                    }

                    // Check pattern
                    if (! dbc.match(mbean, curser.getPattern())) {
                        continue;
                    }

                    // Check server
                    if (curser.getServer() != null && curser.getServer().length() > 0 && !curser.getServer().equals(elem.getServer())) {
                        continue;
                    }

                    // This event matches this Serie and will be kept
                    EventData ed = new EventData(curser.getId(), elem.getTimestamp().getTime(), sValue);
                    dbc.addResult(ed);
                    // continue because an event may match several series
                }
            }

            // Compute the fromDate for next iteration
            // possible to lost a few events here (those not read in the same second)
            fromDate = new Date(lasttimestamp + 1000);

            // replay from database completed if we have reached the time limit
            if (lasttimestamp >= timeLimit) {
                break;
            }
        }
        // replay is completed
        dataBaseEventExtractor.finished();
        dbc.setReplayCompleted();
    }

}
