/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.eos.monitoring.service;

import java.io.File;
import java.io.FilenameFilter;
import java.util.StringTokenizer;

/**
 * Filter used for ReplayFileName
 * @author durieuxp
 *
 */
public class NameFilter implements FilenameFilter {

    private String [] filters = null;

    public NameFilter(final String filter) {
        StringTokenizer st = new StringTokenizer(filter, ",");
        int tokenCount = st.countTokens();
        filters = new String[tokenCount];
        for (int i = 0; i < tokenCount; i++) {
            filters[i] = st.nextToken();
        }
    }

    public boolean accept(final File dir, final String name) {
        // Forget all ~ files.
        if (name.endsWith("~")) {
            return false;
        }
        // Keep only normal files
        File file = new File(dir, name);
        if (! file.isFile()) {
            return false;
        }
        // Loop on filters
        for (int i = 0; i < filters.length; i++) {
            if (matchFile(name, filters[i])) {
                return true;
            }
        }
        return false;
    }

    private boolean matchFile(final String name, final String filter) {
        // Check start
        int first = filter.indexOf('*');
        if (first > 0 && ! name.startsWith(filter.substring(0, first))) {
            return false;
        }
        // check end
        int last = filter.lastIndexOf('*');
        if (last >= 0 && last < filter.length() && ! name.endsWith(filter.substring(last + 1))) {
            return false;
        }
        // TODO better check
        return true;
    }
}
